#ifndef RENDERABLEOBJECTHEADER
#define RENDERABLEOBJECTHEADER
#include <windows.h>
#include <GL/gl.h> 
#include <math.h>
#include <stdio.h>
#include <map>
#include <Vector3.h>
#include <Quaternion.h>
class RenderableObject
{
	public:
		RenderableObject();
		~RenderableObject();
		void Render();
		virtual void Draw();
		static void AllocStatic();
		static void FreeStatic();

		bool operator < (const RenderableObject & obj) const;
		virtual void SetPos(GLfloat x, GLfloat y,GLfloat z);
		virtual void SetPos(const Vector3 & v);
		void GetPos(Vector3 & v) const;
		void SetAngle(GLfloat x, GLfloat y,GLfloat z);

		void SetAngle(const Vector3 & v);
		void SetAngle(const Quaternion & q);

		void SetXform(const Matrix4x4 & m);

		void SetScale(GLfloat x, GLfloat y,GLfloat z);
		void SetScale(const Vector3 & v);
		int GetID() const;

		void AddChild(RenderableObject*r);
		void RemoveChild(int objId);

		void GetLocalXform(Matrix4x4 & m);
		void GetFullXform(Matrix4x4 & m);
		void GetFullXformInv(Matrix4x4 & m);

	private:

	protected:

		void UpdateParent(RenderableObject * p);

		static int nextID;
		int objID;
		void ApplyTransform();
		void ApplyInverseTransform();
		std::map<int,RenderableObject*> Children;
		RenderableObject * parent;
		Vector3 Pos;
		Quaternion Angle;
		Vector3 Scale;
		Matrix4x4 xform;
		Matrix4x4 xforminv;
		bool invDirty;
};


#endif