#include "KeyFrameAnim.h"

std::map<std::string,KeyFrameAnim> KeyFrameAnim::anims;

KeyFrameAnim::KeyFrameAnim()
{
	numJoints = 0;
}

KeyFrameAnim::~KeyFrameAnim()
{
}

bool 
KeyFrameAnim::Load(const char * name, const char * fname)
{
	std::map<std::string,KeyFrameAnim>::iterator it;
	it = anims.find(name);

	//printf anim already loaded (hope its the same file name)!
	if (it != anims.end())
	{
		if (it->second.fname == std::string(fname))
			return true;

		printf("A different animation is already loaded under the name %s\n",fname);
		return false;
	}

	anims[name] = KeyFrameAnim();
	if (!anims[name].LoadFromFile(fname))
	{
		printf("Error loading anim %s\n",name);
		anims.erase(name);
		return false;
	}

	return true;
}

void 
KeyFrameAnim::Unload(const char * name)
{
	std::map<std::string,KeyFrameAnim>::iterator it;
	it = anims.find(name);
	
	if (it != anims.end())
		return;

	anims.erase(it);
}

KeyFrameAnim * 
KeyFrameAnim::GetPtr(const char * name)
{
	std::map<std::string,KeyFrameAnim>::iterator it;
	it = anims.find(name);
	if (it == anims.end())
		return NULL;
	return &it->second;
}


bool 
KeyFrameAnim::LoadFromFile(const char * name)
{
	FILE * f = fopen(name,"rb");
	if (!f)
		return false;
	maxtime = 0;

	char header[5];
	fread(header,5,1,f);
	header[4] = 0;
	if (strncmp(header,"ANIM",4))
	{
		fclose(f);
		return false;
	}
	fname = name;

	fread(&numJoints,1,4,f);
	currentPose = Pose(numJoints);
	tracks.resize(numJoints);	

	for (unsigned int i = 0; i < numJoints;i++)
	{
		char name[21];
		fread(name,sizeof(name),1,f);
		int numkeys;
		
		fread(&numkeys,4,1,f);

		for (int j = 0; j < numkeys; j++)
		{
			float time;
			fread(&time,4,1,f);
			float rot[3];
			
			//should really read in the rotation order. . .
			
			fread(rot,4,3,f);
			Matrix4x4 z, x, y,r;
			z.RotateZ(rot[2]);
			y.RotateY(rot[1]);
			x.RotateX(rot[0]);
			
			if (i == 0)
			{
				Quaternion q;
				q.setAngles(0,rot[1],0);
				heading.AddValue(time,q);
				y.Identity();

				head.AddValue(time,rot[1]);

			}

			r.Mult(y,x);
			r.PostMult(z);
			
			Quaternion q;
			q.setMatrix(r);
			tracks[i].AddValue(time,q);
		}
	}

	//now load in the root translations
	int numkeys;
	fread(&numkeys,4,1,f);

	for (int i = 0; i < numkeys; i++)
	{
		float time;
		fread(&time,4,1,f);
		
		if (time > maxtime)
			maxtime = time;
		float trans[3];
		fread(trans,4,3,f);
		position.AddValue(time,Vector3(trans[0],0,trans[2]));
		height.AddValue(time,trans[1]);
	}

	//calcuate x0 and x0i
	Matrix4x4 r0,x0i;
	Vector3 t0;
	t0 = position.vals[0];
	t0.Mult(-1);
	x0i.Translate(t0);
	heading.vals[0].getMatrix(r0);
	r0.Transpose();
	x0i.PreMult(r0);


	//process the root keys so that they are relative
	
	for(i = 0; i < numkeys; i++)
	{
		position.vals[i].Mult(x0i);
	}
	position.vals[0].Set(0,0,0);

	numkeys = (int)heading.vals.size();
	for (int i = numkeys-1; i >= 0; i--)
	{
		head.vals[i] -= head.vals[0];
		heading.vals[i] = heading.vals[0].Difference(heading.vals[i]);
	}

	fclose(f);
	return true;
}

void 
KeyFrameAnim::SetTime(const float time)
{
	
	//update all of the tracks
	head.SetTime(time);
	heading.SetTime(time);
	height.SetTime(time);
	position.SetTime(time);
	for (unsigned int i = 0; i < numJoints; i++ )
		tracks[i].SetTime(time);
	

	//now do an interpolation to update currentPose
	currentPose.SetHeight(height.GetValue());
	currentPose.SetPosition(position.GetValue());
	
	
	//currentPose.SetHeading(heading.GetValue());
	currentPose.SetHeading(head.GetValue());
	
	for (unsigned int i = 0; i < numJoints; i++)
		currentPose.SetJointRot(i,tracks[i].GetValue());
}
	
unsigned int 
KeyFrameAnim::GetNumJoints() const
{
	return numJoints;
}

Pose
KeyFrameAnim::GetPose()
{
	return currentPose;
}


float
KeyFrameAnim::GetDuration()
{
	return maxtime;
}

void
KeyFrameAnim::GetTranslation(Vector3 & v)
{
	v = translation;
}