#include "ReactiveAgent.h"

ReactiveAgent::ReactiveAgent()
{
	reactmode = false;
	debugcolorr = 1.0;
	debugcolorg = 0.5;
	debugcolorb = 0.0;
	ig.SetAgent(this);
};

ReactiveAgent::~ReactiveAgent()
{
}

void
ReactiveAgent::ScoreAction(ActionTerminator at)
{
	float score = state.FinishAction(at);

	switch (at)
	{
	case HIT:
		score = 1.25f;
		break;
	case OP_HIT:
		score = 0.8f;
		break;
	case BLOCK:
		score = 1.15f;
		break;
	case OP_BLOCK:
		score = 0.875f;
		break;
	case COMPLETE:
		if (currentaction == "jab" || currentaction == "cross" ||
		    currentaction == "hook" || currentaction == "upper_cut")
		{
			score = 0.95f;

			opponent->ScoreAction(DODGE);
		}
		else if (currentaction == "feint" || currentaction == "block" ||
		     currentaction == "dodge_left" || currentaction == "dodge_right" ||
			 currentaction == "dodge_down" || currentaction == "dodge_back")
		{
			if (opponent->GetAction() == "jab" ||
			    opponent->GetAction() == "cross" ||
			    opponent->GetAction() == "hook" ||
			    opponent->GetAction() == "upper_cut")
				score = 1.05f;
			else
				score = 0.975f;
		}
		else if (currentaction == "recover")
		{
			if (state.GetEnergy() > 0.5f)
				score = 0.95f;
			else
				score = 1.025f;
		}
		break;
	case DODGE:
		score = 1.1f;
		break;
	}

	ig.ScorePath(score);

	char str[50];
	sprintf(str, "Energy: %1.2f", state.GetEnergy());
	thoughts[2] = str;
}

void 
ReactiveAgent::Think()
{
	for (unsigned int i = 0; i < events.size(); i++)
	{
		if (events[i].type == InputEvent::KeyDown)
		{
			switch (events[i].ID)
			{
			case '0': case '9':
				reactmode = !reactmode;
				if (reactmode)
				{
					thoughts[0] = "Activating reactive boxer";
				}
				else
				{
					PlayAction("frozen_1");
					thoughts[0] = "Disabling reactive boxer";
				}
				break;
			case '-':
				if (ig.WriteProfile("boxer1.adj"))
					printf("Wrote adjustments file for boxer 1\n");
				else
					printf("File write failed\n");
				break;
			case '=':
				if (ig.WriteProfile("boxer2.adj"))
					printf("Wrote adjustments file for boxer 2\n");
				else
					printf("File write failed\n");
				break;
			}
		}
	}
	events.clear();

	if (reactmode && (actioninterruptible || actor.IsIdle()))
	{
		if (actor.IsIdle())
			ScoreAction(COMPLETE);

		Intention intention = ig.GenerateIntention();
		if (br.PlayIntention(intention))
		{
			switch (intention)
			{
			case FACE:
				thoughts[0] = "Facing opponent";
				break;
			case RECOVER:
				thoughts[0] = "Recovering";
				break;
			case APPROACH:
				thoughts[0] = "Approaching opponent";
				break;
			case EVADE:
				thoughts[0] = "Evading";
				break;
			case ATTACK:
				thoughts[0] = "Attacking";
				break;
			case DEFEND:
				thoughts[0] = "Defending";
				break;
			case ESCAPE_ROPES:
				thoughts[0] = "Escaping ropes";
				break;
			default:
				break;
			}
		}
	}
	else if (!reactmode && actor.IsIdle())
	{
		PlayAction("frozen_1");
		thoughts[0] = "Frozen";
	}

	thoughts[1] = actor.GetCurrentAnimation();

/*	Matrix4x4 myhead;
	Vector3  me(0,0,0);
	actor.GetJointXform("pelvis", myhead);
	me.Mult(myhead);

	float opangle = perception.OpponentAngle();

	char str[50];
	sprintf(str,"Angle: %.2f  xyz: %.2f %.2f %.2f", opangle, me[0], me[1], me[2]);
	
	thoughts[2] = str;*/
}

bool
ReactiveAgent::LoadIGAdjustments(const char * filename)
{
	return ig.LoadProfile(filename);
}

void
ReactiveAgent::ResetIGAdjustments()
{
	ig.Reset();
}