#include "TestScene.h"

//#include "glPrint.h"
#include "Font.h"

void DrawArrow(float sc)
{
	glColor3f(0,1,0);

	
	glBegin(GL_LINES);
	glVertex3f(0,0,0);
	glVertex3f(0,0,sc);
	glVertex3f(0,0,sc);
	glVertex3f(0,sc/8,7*sc/8);
	glVertex3f(0,0,sc);
	glVertex3f(0,-sc/8,7*sc/8);
	glEnd();
}

TestScene::TestScene()
{
}

TestScene::~TestScene()
{
}

bool 
TestScene::Initialize()
{
	initTime = TimeStamp();

	theta = phi = 0;
	dist = 10;
	theCamera.SetDist(dist);
	theCamera.SetOrbitAngle(theta,phi);
	theCamera.Orbit();
	
	//boxer.AddChild(&theCamera);
	overheadLight.SetPos(0,20,0);
	frontLight.SetPos(0,0,5);
	frontLight.SetAmbient(.5,.5,.5);
	//load up some test data
	
//	Repetoire r;
//	r.LoadXML("boxerrep.xml");
	if (!SkelChar::Load("Boxer","test.skel"))
		return false;
	
	
	Material * mat = Material::CreateManagedMaterial("Boxer");
	if (mat)
	{
		mat->setAmbient(1,1,1);
		mat->setTexture("boxer.bmp",true);
	}

	
	
	Font::LoadFont("Tahoma.bmp","tahoma.dat","Tahoma",false);

	
	if (!boxer.LoadXML("boxer.xml"))
		return false;
	
	if (!KeyFrameAnim::Load("tpose","Tpose.anim"))
		return false;

	if (!KeyFrameAnim::Load("jab","jab.anim"))
		return false;

	if (!KeyFrameAnim::Load("huge","huge.anim"))
		return false;

	if (!KeyFrameAnim::Load("uppercut","uppercut.anim"))
		return false;

	if (!KeyFrameAnim::Load("step1","step2.anim"))
		return false;

	if (!KeyFrameAnim::Load("circle","circle.anim"))
		return false;
	
	if (!boxer.SetSkel("Boxer"))
		return false;
	
	boxer.SetMaterial("Boxer");

	
	updates = 0;
	
	return true;
}

void 
TestScene::RenderPersp()
{
	
	
	TimeStamp t1;
	glEnable(GL_LIGHTING);
	
	//look through the default camera
	theCamera.SetView();

	frontLight.Switch(true);
	overheadLight.Switch(true);


	//draw the world coordinate axis
	Axis3D::StaticDraw();

	boxer.Render();
	Material::applyDefault();
	glColor3f(1,1,1);
	Matrix4x4 m;
	Vector3 v;
	glDisable(GL_LIGHTING);

	glPushMatrix();
	boxer.GetJointXform("pelvis",m);
	m.Transpose();
	glMultMatrixf(m.GetPtr());
	DrawArrow(1);
	glPopMatrix();

	glPushMatrix();
	boxer.GetJointXform("head",m);
	m.Transpose();
	glMultMatrixf(m.GetPtr());
	DrawArrow(1);
	glPopMatrix();

	glPushMatrix();
	boxer.GetJointXform("lfoot",m);
	m.Transpose();
	glMultMatrixf(m.GetPtr());
	DrawArrow(1);
	glPopMatrix();

	glEnable(GL_LIGHTING);



	glDisable(GL_LIGHTING);

	renderTime = t1.Interval(TimeStamp());
}

void 
TestScene::RenderOrtho()
{
	static TimeStamp last;
	
	glTranslatef(10,24,0);
	glColor3f(1,1,1);

	Font::Printf("Tahoma",30,false,"FPS:%3.2f",1.0f/last.Interval(TimeStamp()));
	glTranslatef(0,24,0);
	last.Reset();
}

void
TestScene::HandleInput()
{
	std::set<InputEvent>::iterator it;
	for (it = events.begin(); it != events.end(); it++)
	{
		switch(it->type)
		{
		case InputEvent::EventType::KeyDown:
			switch(it->ID)
			{
			case 'q':
				exit(1);
			break;
			case '1':
				boxer.PlayAnim("tpose");
			break;
			case '2':
				boxer.PlayAnim("jab");
			break;
			case '3':
				boxer.PlayAnim("uppercut");
			break;
			case '4':
				boxer.PlayAnim("circle");
			break;
			case '5':
				boxer.PlayAnim("huge");
			break;
			case '6':
				boxer.PlayAnim("step1");
			break;

			default:
				break;
			}
			break;
		case InputEvent::EventType::MouseDrag:
			theta += it->coordinate.Get(0)*.01f;
			phi += it->coordinate.Get(1)*.01f;
			dist += it->coordinate.Get(2)*.1f;
			theCamera.SetOrbitAngle(theta,phi);
			theCamera.SetDist(dist);
			theCamera.Orbit();
			break;

		default:
			break;

		}
	}
	events.clear();
}


void 
TestScene::Update()
{


	HandleInput();

	TimeStamp t1;
	interval = lastUpdate.Interval(t1);
	lastUpdate = t1;
	updates++;

	
	theCamera.SetOrbitAngle((float)t1.Msec()/1000.0f,1.0f );
	boxer.Update();
	Vector3 lfv,rfv,bp,pv;
	Matrix4x4 lfm,rfm,pm;
	
/*	boxer.GetJointXform("lfoot",lfm);
	boxer.GetJointXform("rfoot",rfm);
	boxer.GetJointXform("pelvis",pm);
	lfv.Mult(lfm);
	rfv.Mult(rfm);
	pv.Mult(pm);

	float offset = pv.Get(1) - min(lfv.Get(1),rfv.Get(1));
	boxer.GetPos(bp);
	bp.Set(offset,1);
	boxer.SetPos(bp);
*/
	updateTime = t1.Interval(TimeStamp());
}
