#include "Vector3.h"
#include <assert.h>
#include <limits>

Vector3::Vector3()
{
	data[0] = data[1] = data[2] = 0.0f;
}

Vector3::~Vector3()
{
}

Vector3::Vector3(const float* f)
{
	data[0] = f[0];
	data[1] = f[1];
	data[2] = f[2];
}

Vector3::Vector3(const float a, const float b, const float c)
{
	data[0] = a;
	data[1] = b;
	data[2] = c;
}


void 
Vector3::Interpolate(const Vector3 & v1, const Vector3 & v2, const float blend)
{
	float b = blend;
	if (b < 0.0f)
		b = 0.0f;
	if (b > 1.0f)
		b = 1.0f;
	
	data[0] = (1.0f-b)*v1.data[0] + b*v2.data[0];
	data[1] = (1.0f-b)*v1.data[1] + b*v2.data[1];
	data[2] = (1.0f-b)*v1.data[2] + b*v2.data[2];
}


	
void 
Vector3::Normalize()
{
	float mag = sqrtf(data[0]*data[0]+data[1]*data[1]+data[2]*data[2]);
	if (mag > 0.0f)
	{
		data[0] /= mag;
		data[1] /= mag;
		data[2] /= mag;
	}
}

	
void 
Vector3::Cross(const Vector3 & v)
{
	float x = data[1]*v.data[2] - data[2]*v.data[1];
	float y = data[2]*v.data[0] - data[0]*v.data[2];
	float z = data[0]*v.data[1] - data[1]*v.data[0];
	data[0] = x;
	data[1] = y;
	data[2] = z;
}


void 
Vector3::Cross(const Vector3 & v1,const Vector3 & v2)
{
	data[0] = v1.data[1]*v2.data[2] - v1.data[2]*v2.data[1];
	data[1] = v1.data[2]*v2.data[0] - v1.data[0]*v2.data[2];
	data[2] = v1.data[0]*v2.data[1] - v1.data[1]*v2.data[0];
}

void 
Vector3::Add(const Vector3 & v1,const Vector3 & v2)
{
	data[0] = v1.data[0]+v2.data[0];
	data[1] = v1.data[1]+v2.data[1];
	data[2] = v1.data[2]+v2.data[2];
}
	
void 
Vector3::Sub(const Vector3 & v)
{
	data[0] -= v.data[0];
	data[1] -= v.data[1];
	data[2] -= v.data[2];
}

void 
Vector3::Sub(const Vector3 & v1,const Vector3 & v2)
{
	data[0] = v1.data[0]-v2.data[0];
	data[1] = v1.data[1]-v2.data[1];
	data[2] = v1.data[2]-v2.data[2];
}

float 
Vector3::Distance(const Vector3 & v) const
{
	float dx = data[0]-v.data[0];
	float dy = data[1]-v.data[1];
	float dz = data[2]-v.data[2];
	return sqrtf(dx*dx+dy*dy+dz*dz);
}


void 
Vector3::Print(FILE * f) const
{
	fprintf(f, "|%.2f\t%.2f\t%.2f\t1.0|",data[0],data[1],data[2]);
}

Vector3 
operator*(const float f, const Vector3 & v)
{
	Vector3 r;
	r.data[0] = v.data[0]*f;
	r.data[1] = v.data[1]*f;
	r.data[2] = v.data[2]*f;
	return r;
}
